# -*- coding: utf-8 -*-
from __future__ import unicode_literals, print_function, absolute_import

from django.core.management import BaseCommand, CommandError

from gum.indexer import indexer

try:
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import get_model


class Command(BaseCommand):

    help = "Command for managing indices from Elasticsearch"

    def add_arguments(self, parser):
        actions = parser.add_mutually_exclusive_group()
        actions.add_argument(
            "--update-index", nargs="*", action="store", dest='update-index', help="updates indices for given models"
        )
        actions.add_argument(
            "--update-settings", action="store_const", dest="action",  const="update-settings", help="updates settings",
        )
        actions.add_argument(
            "--remove", action="store_const", dest="action", const="remove", help="remove all indices"
        )

    def _update_index(self, models):
        """Order to update index."""
        selected_models = []
        for input_model in models:
            try:
                app_name, model_name = input_model.split(".")
                model_class = get_model(app_name, model_name)
                selected_models.append(model_class)
            except (ValueError, LookupError):
                raise CommandError("Model not found for indexing")
        restrict_to = selected_models if selected_models else None

        self.stdout.write('Initializing index...', ending='')
        indexer.initialize_index()
        self.stdout.write(' OK')
        indexer.update_index(stdout=self.stdout, restrict_to=restrict_to)

    def _update_settings(self):
        """Order to update settings."""
        self.stdout.write('Updating index settings... ', ending='')
        indexer.update_settings()
        self.stdout.write(' OK')

    def _remove_index(self):
        """Order to remove index."""
        self.stdout.write('Removing index... ', ending='')
        indexer.remove_index()
        self.stdout.write(' OK')

    def handle(self, *args, **options):
        actions = {
            "update-settings", self._update_settings,
            "remove", self._remove_index,
        }
        action = options.get("action")
        if action in actions:
            actions[options.get("action")]()
        if "update-index" in options:
            self._update_index(options.get("update-index"))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import django


def autodiscover():
    """
    Auto-discover INSTALLED_APPS translation.py modules and fail silently when
    not present. This forces an import on them to register.
    Also import explicit modules.
    """
    import os
    import sys
    import copy
    from django.conf import settings
    from django.utils.module_loading import module_has_submodule
    from gum.indexer import indexer
    from gum.settings import INDEX_FILES, DEBUG

    if django.VERSION < (1, 7):
        from django.utils.importlib import import_module
        mods = [(app, import_module(app)) for app in settings.INSTALLED_APPS]
    else:
        from importlib import import_module
        from django.apps import apps
        mods = [(app_config.name, app_config.module) for app_config in apps.get_app_configs()]

    for (app, mod) in mods:
        # Attempt to import the app's translation module.
        module = '%s.index' % app
        before_import_registry = copy.copy(indexer._registry)
        try:
            import_module(module)
        except:
            # Reset the model registry to the state before the last import as
            # this import will have to reoccur on the next request and this
            # could raise NotRegistered and AlreadyRegistered exceptions
            indexer._registry = before_import_registry

            # Decide whether to bubble up this error. If the app just
            # doesn't have an translation module, we can ignore the error
            # attempting to import it, otherwise we want it to bubble up.
            if module_has_submodule(mod, 'index'):
                raise

    for module in INDEX_FILES:
        import_module(module)

    # In debug mode, print a list of registered models and pid to stdout.
    # Note: Differing model order is fine, we don't rely on a particular
    # order, as far as base classes are registered before subclasses.
    if DEBUG:
        try:
            if sys.argv[1] in ('runserver', 'runserver_plus'):
                models = indexer.get_registered_models()
                names = ', '.join(m.__name__ for m in models)
                print('gum: Registered %d models for indexing'
                      ' (%s) [pid: %d].' % (len(models), names, os.getpid()))
        except IndexError:
            pass


def handle_indexer_registrations(*args, **kwargs):
    autodiscover()


if django.VERSION < (1, 7):
    handle_indexer_registrations()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from gum.utils import elasticsearch_connection


class ElasticsearchManager(object):
    """Like a `ModelManager` gives to the user methods to apply queries
    to Elasticsearch from a specific model.
    """

    def __init__(self, model=None, mapping_type=None):
        self.model = model
        self.mapping_type = mapping_type

    def search(self, **kwargs):
        """Partial application of `search` function from Elasticsearch
        module.

        :param kwargs:
        """
        es = elasticsearch_connection()
        if 'index' not in kwargs:
            kwargs["index"] = self.mapping_type.index
        if 'doc_type' not in kwargs:
            kwargs["doc_type"] = self.mapping_type.get_type()
        return es.search(**kwargs)

    def index(self, instance):
        """Shortcut to index an instance.

        :param instance:
        :return:
        """
        return self.mapping_type.index_document(instance)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from elasticsearch import Elasticsearch

from gum.settings import ELASTICSEARCH_URLS, ELASTICSEARCH_TIMEOUT, ELASTICSEARCH_CONNECTION_PARAMS


def _build_key(urls):
    key = (urls,)
    return key


_cached_elasticsearch = {}


def elasticsearch_connection(urls=None):
    """Create an elasticsearch `Elasticsearch` object and return it.
    :param urls:
    """
    connection_urls = urls or ELASTICSEARCH_URLS
    key = _build_key(urls)
    if key in _cached_elasticsearch:
        return _cached_elasticsearch[key]
    try:
        _ = iter(connection_urls)
    except TypeError:
        connection_urls = [connection_urls]
    es_params = ELASTICSEARCH_CONNECTION_PARAMS
    es = Elasticsearch(hosts=connection_urls, **es_params)
    _cached_elasticsearch[key] = es
    return es
